@@ -9,8 +9,7 @@ variable. To enable threading resolver:
 
     GEVENT_RESOLVER=thread python dns_mass_resolve.py
 """
-from __future__ import with_statement
-import sys
+from __future__ import print_function
 import gevent
 from gevent import socket
 from gevent.pool import Pool
@@ -26,16 +25,15 @@ def job(url):
     try:
         try:
             ip = socket.gethostbyname(url)
-            print ('%s = %s' % (url, ip))
-        except socket.gaierror:
-            ex = sys.exc_info()[1]
-            print ('%s failed with %s' % (url, ex))
+            print('%s = %s' % (url, ip))
+        except socket.gaierror as ex:
+            print('%s failed with %s' % (url, ex))
     finally:
         finished += 1
 
 with gevent.Timeout(2, False):
-    for x in xrange(10, 10 + N):
+    for x in range(10, 10 + N):
         pool.spawn(job, '%s.com' % x)
     pool.join()
 
-print ('finished within 2 seconds: %s/%s' % (finished, N))
+print('finished within 2 seconds: %s/%s' % (finished, N))
