@@ -1,17 +1,18 @@
 #!/usr/bin/python
 """WSGI server example"""
+from __future__ import print_function
 from gevent.pywsgi import WSGIServer
 
 
 def application(env, start_response):
     if env['PATH_INFO'] == '/':
         start_response('200 OK', [('Content-Type', 'text/html')])
-        return ["<b>hello world</b>"]
+        return [b"<b>hello world</b>"]
     else:
         start_response('404 Not Found', [('Content-Type', 'text/html')])
-        return ['<h1>Not Found</h1>']
+        return [b'<h1>Not Found</h1>']
 
 
 if __name__ == '__main__':
-    print 'Serving on 8088...'
+    print('Serving on 8088...')
     WSGIServer(('', 8088), application).serve_forever()
