@@ -4,7 +4,7 @@ import sys
 import _socket
 from gevent.baseserver import BaseServer
 from gevent.socket import EWOULDBLOCK, socket
-
+from gevent.hub import PYPY, PY3
 
 __all__ = ['StreamServer', 'DatagramServer']
 
@@ -38,6 +38,8 @@ class StreamServer(BaseServer):
 
     The delay starts with :attr:`min_delay` and doubles with each successive error until it reaches :attr:`max_delay`.
     A successful :func:`accept` resets the delay to :attr:`min_delay` again.
+
+    See :class:`BaseServer` for information on defining the *handle* function and important restrictions on it.
     """
     # the default backlog to use if none was provided in __init__
     backlog = 256
@@ -88,14 +90,36 @@ class StreamServer(BaseServer):
             backlog = self.backlog
         return _tcp_listener(address, backlog=backlog, reuse_addr=self.reuse_addr, family=family)
 
-    def do_read(self):
-        try:
-            client_socket, address = self.socket.accept()
-        except _socket.error, err:
-            if err[0] == EWOULDBLOCK:
-                return
-            raise
-        return socket(_sock=client_socket), address
+    if PY3:
+
+        def do_read(self):
+            sock = self.socket
+            try:
+                fd, address = sock._accept()
+            except BlockingIOError:
+                if not sock.timeout:
+                    return
+                raise
+            sock = socket(sock.family, sock.type, sock.proto, fileno=fd)
+            # XXX Python issue #7995?
+            return sock, address
+
+    else:
+
+        def do_read(self):
+            try:
+                client_socket, address = self.socket.accept()
+            except _socket.error as err:
+                if err.args[0] == EWOULDBLOCK:
+                    return
+                raise
+            sockobj = socket(_sock=client_socket)
+            if PYPY:
+                client_socket._drop()
+            return sockobj, address
+
+    def do_close(self, socket, *args):
+        socket.close()
 
     def wrap_socket_and_handle(self, client_socket, address):
         # used in case of ssl sockets
@@ -130,8 +154,8 @@ class DatagramServer(BaseServer):
     def do_read(self):
         try:
             data, address = self._socket.recvfrom(8192)
-        except _socket.error, err:
-            if err[0] == EWOULDBLOCK:
+        except _socket.error as err:
+            if err.args[0] == EWOULDBLOCK:
                 return
             raise
         return data, address
@@ -151,8 +175,7 @@ def _tcp_listener(address, backlog=50, reuse_addr=None, family=_socket.AF_INET):
         sock.setsockopt(_socket.SOL_SOCKET, _socket.SO_REUSEADDR, reuse_addr)
     try:
         sock.bind(address)
-    except _socket.error:
-        ex = sys.exc_info()[1]
+    except _socket.error as ex:
         strerror = getattr(ex, 'strerror', None)
         if strerror is not None:
             ex.strerror = strerror + ': ' + repr(address)
@@ -169,8 +192,7 @@ def _udp_socket(address, backlog=50, reuse_addr=None, family=_socket.AF_INET):
         sock.setsockopt(_socket.SOL_SOCKET, _socket.SO_REUSEADDR, reuse_addr)
     try:
         sock.bind(address)
-    except _socket.error:
-        ex = sys.exc_info()[1]
+    except _socket.error as ex:
         strerror = getattr(ex, 'strerror', None)
         if strerror is not None:
             ex.strerror = strerror + ': ' + repr(address)
