@@ -1,23 +1,24 @@
 # Copyright (c) 2012 Denis Bilenko. See LICENSE for details.
-from __future__ import with_statement, absolute_import
+from __future__ import absolute_import
 import sys
 import os
 from gevent.hub import get_hub, getcurrent, sleep, integer_types
 from gevent.event import AsyncResult
 from gevent.greenlet import Greenlet
-from gevent.pool import IMap, IMapUnordered
+from gevent.pool import GroupMappingMixin
 from gevent.lock import Semaphore
 from gevent._threading import Lock, Queue, start_new_thread
 
-# XXX apply_e is ugly and must not be needed
-# XXX apply() should re-raise everything
-
 
 __all__ = ['ThreadPool',
            'ThreadResult']
 
 
-class ThreadPool(object):
+class ThreadPool(GroupMappingMixin):
+    """
+    .. note:: The method :meth:`apply_async` will always return a new
+       greenlet, bypassing the threadpool entirely.
+    """
 
     def __init__(self, maxsize, hub=None):
         if hub is None:
@@ -89,7 +90,8 @@ class ThreadPool(object):
 
     def _on_fork(self):
         # fork() only leaves one thread; also screws up locks;
-        # let's re-create locks and threads
+        # let's re-create locks and threads.
+        # NOTE: See comment in gevent.hub.reinit.
         pid = os.getpid()
         if pid != self.pid:
             self.pid = pid
@@ -99,6 +101,7 @@ class ThreadPool(object):
             self._init(self._maxsize)
 
     def join(self):
+        """Waits until all outstanding tasks have been completed."""
         delay = 0.0005
         while self.task_queue.unfinished_tasks > 0:
             sleep(delay)
@@ -146,22 +149,33 @@ class ThreadPool(object):
             raise
 
     def spawn(self, func, *args, **kwargs):
+        """
+        Add a new task to the threadpool that will run ``func(*args, **kwargs)``.
+
+        Waits until a slot is available. Creates a new thread if necessary.
+
+        :return: A :class:`gevent.event.AsyncResult`.
+        """
         while True:
             semaphore = self._semaphore
             semaphore.acquire()
             if semaphore is self._semaphore:
                 break
+
+        thread_result = None
         try:
             task_queue = self.task_queue
             result = AsyncResult()
-            thread_result = ThreadResult(result, hub=self.hub)
+            # XXX We're calling the semaphore release function in the hub, otherwise
+            # we get LoopExit (why?). Previously it was done with a rawlink on the
+            # AsyncResult and the comment that it is "competing for order with get(); this is not
+            # good, just make ThreadResult release the semaphore before doing anything else"
+            thread_result = ThreadResult(result, hub=self.hub, call_when_ready=semaphore.release)
             task_queue.put((func, args, kwargs, thread_result))
             self.adjust()
-            # rawlink() must be the last call
-            result.rawlink(lambda *args: self._semaphore.release())
-            # XXX this _semaphore.release() is competing for order with get()
-            # XXX this is not good, just make ThreadResult release the semaphore before doing anything else
         except:
+            if thread_result is not None:
+                thread_result.destroy()
             semaphore.release()
             raise
         return result
@@ -188,21 +202,21 @@ class ThreadPool(object):
                         # otherwise, _adjust might think there's one more idle thread that
                         # needs to be killed
                         return
-                    func, args, kwargs, result = task
+                    func, args, kwargs, thread_result = task
                     try:
                         value = func(*args, **kwargs)
                     except:
                         exc_info = getattr(sys, 'exc_info', None)
                         if exc_info is None:
                             return
-                        result.handle_error((self, func), exc_info())
+                        thread_result.handle_error((self, func), exc_info())
                     else:
                         if sys is None:
                             return
-                        result.set(value)
+                        thread_result.set(value)
                         del value
                     finally:
-                        del func, args, kwargs, result, task
+                        del func, args, kwargs, thread_result, task
                 finally:
                     if sys is None:
                         return
@@ -211,119 +225,111 @@ class ThreadPool(object):
             if need_decrease:
                 self._decrease_size()
 
-    # XXX apply() should re-raise error by default
-    # XXX because that's what builtin apply does
-    # XXX check gevent.pool.Pool.apply and multiprocessing.Pool.apply
     def apply_e(self, expected_errors, function, args=None, kwargs=None):
-        if args is None:
-            args = ()
-        if kwargs is None:
-            kwargs = {}
-        success, result = self.spawn(wrap_errors, expected_errors, function, args, kwargs).get()
-        if success:
-            return result
-        raise result
-
-    def apply(self, func, args=None, kwds=None):
-        """Equivalent of the apply() builtin function. It blocks till the result is ready."""
-        if args is None:
-            args = ()
-        if kwds is None:
-            kwds = {}
-        return self.spawn(func, *args, **kwds).get()
-
-    def apply_cb(self, func, args=None, kwds=None, callback=None):
-        result = self.apply(func, args, kwds)
-        if callback is not None:
-            callback(result)
-        return result
-
-    def apply_async(self, func, args=None, kwds=None, callback=None):
-        """A variant of the apply() method which returns a Greenlet object.
-
-        If callback is specified then it should be a callable which accepts a single argument. When the result becomes ready
-        callback is applied to it (unless the call failed)."""
-        if args is None:
-            args = ()
-        if kwds is None:
-            kwds = {}
-        return Greenlet.spawn(self.apply_cb, func, args, kwds, callback)
-
-    def map(self, func, iterable):
-        return list(self.imap(func, iterable))
-
-    def map_cb(self, func, iterable, callback=None):
-        result = self.map(func, iterable)
-        if callback is not None:
-            callback(result)
-        return result
-
-    def map_async(self, func, iterable, callback=None):
         """
-        A variant of the map() method which returns a Greenlet object.
-
-        If callback is specified then it should be a callable which accepts a
-        single argument.
+        .. deprecated:: 1.1a2
+           Identical to :meth:`apply`; the ``expected_errors`` argument is ignored.
         """
-        return Greenlet.spawn(self.map_cb, func, iterable, callback)
+        # Deprecated but never documented. In the past, before
+        # self.apply() allowed all errors to be raised to the caller,
+        # expected_errors allowed a caller to specify a set of errors
+        # they wanted to be raised, through the wrap_errors function.
+        # In practice, it always took the value Exception or
+        # BaseException.
+        return self.apply(function, args, kwargs)
 
-    def imap(self, func, iterable):
-        """An equivalent of itertools.imap()"""
-        return IMap.spawn(func, iterable, spawn=self.spawn)
+    def _apply_immediately(self):
+        # If we're being called from a different thread than the one that
+        # created us, e.g., because a worker task is trying to use apply()
+        # recursively, we have no choice but to run the task immediately;
+        # if we try to AsyncResult.get() in the worker thread, it's likely to have
+        # nothing to switch to and lead to a LoopExit.
+        return get_hub() is not self.hub
 
-    def imap_unordered(self, func, iterable):
-        """The same as imap() except that the ordering of the results from the
-        returned iterator should be considered in arbitrary order."""
-        return IMapUnordered.spawn(func, iterable, spawn=self.spawn)
+    def _apply_async_cb_spawn(self, callback, result):
+        callback(result)
+
+    def _apply_async_use_greenlet(self):
+        # Always go to Greenlet because our self.spawn uses threads
+        return True
 
 
 class ThreadResult(object):
 
-    def __init__(self, receiver, hub=None):
+    exc_info = ()
+    _call_when_ready = None
+
+    def __init__(self, receiver, hub=None, call_when_ready=None):
         if hub is None:
             hub = get_hub()
         self.receiver = receiver
         self.hub = hub
         self.value = None
         self.context = None
-        self.exc_info = None
         self.async = hub.loop.async()
         self.async.start(self._on_async)
+        if call_when_ready:
+            self._call_when_ready = call_when_ready
+
+    @property
+    def exception(self):
+        return self.exc_info[1] if self.exc_info else None
 
     def _on_async(self):
         self.async.stop()
+        if self._call_when_ready:
+            # Typically this is pool.semaphore.release and we have to
+            # call this in the Hub; if we don't we get the dreaded
+            # LoopExit (XXX: Why?)
+            self._call_when_ready()
         try:
-            if self.exc_info is not None:
-                try:
-                    self.hub.handle_error(self.context, *self.exc_info)
-                finally:
-                    self.exc_info = None
+            if self.exc_info:
+                self.hub.handle_error(self.context, *self.exc_info)
             self.context = None
             self.async = None
             self.hub = None
+            self._call_when_ready = None
             if self.receiver is not None:
-                # XXX exception!!!?
                 self.receiver(self)
         finally:
             self.receiver = None
             self.value = None
+            if self.exc_info:
+                self.exc_info = (self.exc_info[0], self.exc_info[1], None)
+
+    def destroy(self):
+        if self.async is not None:
+            self.async.stop()
+        self.async = None
+        self.context = None
+        self.hub = None
+        self._call_when_ready = None
+        self.receiver = None
+
+    def _ready(self):
+        if self.async is not None:
+            self.async.send()
 
     def set(self, value):
         self.value = value
-        self.async.send()
+        self._ready()
 
     def handle_error(self, context, exc_info):
         self.context = context
         self.exc_info = exc_info
-        self.async.send()
+        self._ready()
 
     # link protocol:
     def successful(self):
-        return True
+        return self.exception is None
 
 
 def wrap_errors(errors, function, args, kwargs):
+    """
+    .. deprecated:: 1.1a2
+       Previously used by ThreadPool.apply_e.
+    """
     try:
         return True, function(*args, **kwargs)
-    except errors:
-        return False, sys.exc_info()[1]
+    except errors as ex:
+        return False, ex
