@@ -1,15 +1,19 @@
 """
 Various tests for synchronization primitives.
 """
-from __future__ import with_statement
-
 import sys
 import time
-from thread import start_new_thread, get_ident
+try:
+    from thread import start_new_thread, get_ident
+except ImportError:
+    from _thread import start_new_thread, get_ident
 import threading
 import unittest
 
-from test import test_support as support
+try:
+    from test import support
+except ImportError:
+    from test import test_support as support
 
 
 def _wait():
@@ -40,7 +44,7 @@ class Bunch(object):
                 self.finished.append(tid)
                 while not self._can_exit:
                     _wait()
-        for i in range(n):
+        for _ in range(n):
             start_new_thread(task, ())
 
     def wait_for_started(self):
@@ -69,6 +73,9 @@ class BaseLockTests(BaseTestCase):
     Tests for both recursive and non-recursive locks.
     """
 
+    def locktype(self):
+        raise NotImplementedError()
+
     def test_constructor(self):
         lock = self.locktype()
         del lock
@@ -240,6 +247,9 @@ class EventTests(BaseTestCase):
     Tests for Event objects.
     """
 
+    def eventtype(self):
+        raise NotImplementedError()
+
     def test_is_set(self):
         evt = self.eventtype()
         self.assertFalse(evt.is_set())
@@ -312,6 +322,9 @@ class ConditionTests(BaseTestCase):
     Tests for condition variables.
     """
 
+    def condtype(self, *args):
+        raise NotImplementedError()
+
     def test_acquire(self):
         cond = self.condtype()
         # Be default we have an RLock: the condition can be acquired multiple
@@ -417,9 +430,13 @@ class BaseSemaphoreTests(BaseTestCase):
     Common tests for {bounded, unbounded} semaphore objects.
     """
 
+    def semtype(self, *args):
+        raise NotImplementedError()
+
     def test_constructor(self):
         self.assertRaises(ValueError, self.semtype, value = -1)
-        self.assertRaises(ValueError, self.semtype, value = -sys.maxint)
+        # Py3 doesn't have sys.maxint
+        self.assertRaises(ValueError, self.semtype, value = -getattr(sys, 'maxint', getattr(sys, 'maxsize', None)))
 
     def test_acquire(self):
         sem = self.semtype(1)
@@ -454,13 +471,13 @@ class BaseSemaphoreTests(BaseTestCase):
             _wait()
         self.assertEqual(results1 + results2, [0] * 6)
         phase_num = 1
-        for i in range(7):
+        for _ in range(7):
             sem.release()
         while len(results1) + len(results2) < 13:
             _wait()
         self.assertEqual(sorted(results1 + results2), [0] * 6 + [1] * 7)
         phase_num = 2
-        for i in range(6):
+        for _ in range(6):
             sem.release()
         while len(results1) + len(results2) < 19:
             _wait()
@@ -549,3 +566,6 @@ class BoundedSemaphoreTests(BaseSemaphoreTests):
         sem.acquire()
         sem.release()
         self.assertRaises(ValueError, sem.release)
+
+if __name__ == '__main__':
+    print("This module contains no tests; it is used by other test cases like test_threading_2")
