@@ -19,11 +19,11 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-from __future__ import with_statement
 import sys
 import greentest
 import weakref
 import time
+import gc
 from gevent import sleep, Timeout
 DELAY = 0.04
 
@@ -34,6 +34,7 @@ class Error(Exception):
 
 class Test(greentest.TestCase):
 
+    @greentest.skipOnAppVeyor("Timing is flaky, especially under Py 3.4/64-bit")
     def test_api(self):
         # Nothing happens if with-block finishes before the timeout expires
         t = Timeout(DELAY * 2)
@@ -48,9 +49,8 @@ class Test(greentest.TestCase):
         # An exception will be raised if it's not
         try:
             with Timeout(DELAY) as t:
-                sleep(DELAY * 2)
-        except Timeout:
-            ex = sys.exc_info()[1]
+                sleep(DELAY * 10)
+        except Timeout as ex:
             assert ex is t, (ex, t)
         else:
             raise AssertionError('must raise Timeout')
@@ -58,15 +58,14 @@ class Test(greentest.TestCase):
         # You can customize the exception raised:
         try:
             with Timeout(DELAY, IOError("Operation takes way too long")):
-                sleep(DELAY * 2)
-        except IOError:
-            ex = sys.exc_info()[1]
+                sleep(DELAY * 10)
+        except IOError as ex:
             assert str(ex) == "Operation takes way too long", repr(ex)
 
         # Providing classes instead of values should be possible too:
         try:
             with Timeout(DELAY, ValueError):
-                sleep(DELAY * 2)
+                sleep(DELAY * 10)
         except ValueError:
             pass
 
@@ -75,7 +74,7 @@ class Test(greentest.TestCase):
         except:
             try:
                 with Timeout(DELAY, sys.exc_info()[0]):
-                    sleep(DELAY * 2)
+                    sleep(DELAY * 10)
                     raise AssertionError('should not get there')
                 raise AssertionError('should not get there')
             except ZeroDivisionError:
@@ -94,7 +93,7 @@ class Test(greentest.TestCase):
         with Timeout(XDELAY, False):
             sleep(XDELAY * 2)
         delta = (time.time() - start)
-        assert delta < XDELAY * 2, delta
+        self.assertTimeWithinRange(delta, 0, XDELAY * 2)
 
         # passing None as seconds disables the timer
         with Timeout(None):
@@ -107,31 +106,30 @@ class Test(greentest.TestCase):
         with Timeout(DELAY * 2, err):
             sleep(DELAY)
         del err
+        gc.collect()
         assert not err_ref(), repr(err_ref())
 
     def test_nested_timeout(self):
         with Timeout(DELAY, False):
-            with Timeout(DELAY * 2, False):
-                sleep(DELAY * 3)
+            with Timeout(DELAY * 10, False):
+                sleep(DELAY * 3 * 20)
             raise AssertionError('should not get there')
 
         with Timeout(DELAY) as t1:
-            with Timeout(DELAY * 2) as t2:
+            with Timeout(DELAY * 20) as t2:
                 try:
-                    sleep(DELAY * 3)
-                except Timeout:
-                    ex = sys.exc_info()[1]
+                    sleep(DELAY * 30)
+                except Timeout as ex:
                     assert ex is t1, (ex, t1)
                 assert not t1.pending, t1
                 assert t2.pending, t2
             assert not t2.pending, t2
 
-        with Timeout(DELAY * 2) as t1:
+        with Timeout(DELAY * 20) as t1:
             with Timeout(DELAY) as t2:
                 try:
-                    sleep(DELAY * 3)
-                except Timeout:
-                    ex = sys.exc_info()[1]
+                    sleep(DELAY * 30)
+                except Timeout as ex:
                     assert ex is t2, (ex, t2)
                 assert t1.pending, t1
                 assert not t2.pending, t2
