@@ -31,20 +31,15 @@ class Test(greentest.TestCase):
         # test that args can be changed later
         io.args = (1, 2, 3)
         # test that only tuple and None are accepted by 'args' attribute
-        try:
-            io.args = 5
-            raise AssertionError('"io.args = 5" must raise TypeError')
-        except TypeError:
-            pass
+        self.assertRaises(TypeError, setattr, io, 'args', 5)
         self.assertEqual(io.args, (1, 2, 3))
-        try:
-            io.args = [4, 5]
-            raise AssertionError('"io.args = [4, 5]" must raise TypeError')
-        except TypeError:
-            pass
+
+        self.assertRaises(TypeError, setattr, io, 'args', [4, 5])
         self.assertEqual(io.args, (1, 2, 3))
         # None also works, means empty tuple
+        # XXX why?
         io.args = None
+        self.assertEqual(io.args, None)
         start = core.time()
         loop.run()
         took = core.time() - start
