@@ -1,4 +1,4 @@
-from greentest import walk_modules, BaseTestCase, main
+from greentest import walk_modules, BaseTestCase, main, NON_APPLICABLE_SUFFIXES
 import six
 
 
@@ -10,10 +10,9 @@ def make_exec_test(path, module):
 
     def test(self):
         #sys.stderr.write('%s %s\n' % (module, path))
-        f = open(path)
-        src = f.read()
-        f.close()
-        six.exec_(src, {})
+        with open(path, 'rb') as f:
+            src = f.read()
+        six.exec_(src, {'__file__': path})
 
     name = "test_" + module.replace(".", "_")
     test.__name__ = name
@@ -21,6 +20,14 @@ def make_exec_test(path, module):
 
 
 for path, module in walk_modules():
+    ignored = False
+    for x in NON_APPLICABLE_SUFFIXES:
+        if module.endswith(x):
+            ignored = True
+            break
+    if ignored:
+        continue
+
     make_exec_test(path, module)
 
 
