@@ -32,7 +32,7 @@ DELAY = 0.01
 greentest.TestCase.error_fatal = False
 
 
-class ExpectedError(Exception):
+class ExpectedError(greentest.ExpectedException):
     pass
 
 
@@ -266,6 +266,7 @@ class TestStuff(greentest.TestCase):
         self.assertEqual(e.get(), 1)
 
     def test_wait_error(self):
+
         def x():
             sleep(DELAY)
             return 1
@@ -274,6 +275,7 @@ class TestStuff(greentest.TestCase):
         self.assertRaises(ExpectedError, gevent.joinall, [x, y], raise_error=True)
         self.assertRaises(ExpectedError, gevent.joinall, [y], raise_error=True)
         x.join()
+    test_wait_error.ignore_leakcheck = True
 
     def test_joinall_exception_order(self):
         # if there're several exceptions raised, the earliest one must be raised by joinall
@@ -284,10 +286,31 @@ class TestStuff(greentest.TestCase):
         b = gevent.spawn(lambda: getcurrent().throw(ExpectedError('second')))
         try:
             gevent.joinall([a, b], raise_error=True)
-        except ExpectedError:
-            ex = sys.exc_info()[1]
+        except ExpectedError as ex:
             assert 'second' in str(ex), repr(str(ex))
         gevent.joinall([a, b])
+    test_joinall_exception_order.ignore_leakcheck = True
+
+    def test_joinall_count_raise_error(self):
+        # When joinall is asked not to raise an error, the 'count' param still
+        # works.
+        def raises_but_ignored():
+            raise ExpectedError("count")
+
+        def sleep_forever():
+            while True:
+                sleep(0.1)
+
+        sleeper = gevent.spawn(sleep_forever)
+        raiser = gevent.spawn(raises_but_ignored)
+
+        gevent.joinall([sleeper, raiser], raise_error=False, count=1)
+        assert_ready(raiser)
+        assert_not_ready(sleeper)
+
+        # Clean up our mess
+        sleeper.kill()
+        assert_ready(sleeper)
 
     def test_multiple_listeners_error(self):
         # if there was an error while calling a callback
@@ -453,6 +476,28 @@ class TestJoinAll(greentest.GenericWaitTestCase):
 
 class TestBasic(greentest.TestCase):
 
+    def test_spawn_non_callable(self):
+        self.assertRaises(TypeError, gevent.spawn, 1)
+        self.assertRaises(TypeError, gevent.spawn_raw, 1)
+
+        # Not passing the run argument, just the seconds argument
+        self.assertRaises(TypeError, gevent.spawn_later, 1)
+        # Passing both, but not implemented
+        self.assertRaises(TypeError, gevent.spawn_later, 1, 1)
+
+    def test_spawn_raw_kwargs(self):
+        value = []
+
+        def f(*args, **kwargs):
+            value.append(args)
+            value.append(kwargs)
+
+        g = gevent.spawn_raw(f, 1, name='value')
+        gevent.sleep(0.01)
+        assert not g
+        self.assertEqual(value[0], (1,))
+        self.assertEqual(value[1], {'name': 'value'})
+
     def test_simple_exit(self):
         link_test = []
 
@@ -461,7 +506,7 @@ class TestBasic(greentest.TestCase):
             return return_value
 
         g = gevent.Greenlet(func, 0.01, return_value=5)
-        g.link(lambda x: link_test.append(x))
+        g.rawlink(link_test.append) # use rawlink to avoid timing issues on Appveyor (not always successful)
         assert not g, bool(g)
         assert not g.dead
         assert not g.started
@@ -497,7 +542,7 @@ class TestBasic(greentest.TestCase):
         assert g.successful()
         assert g.value == 5
         assert g.exception is None  # not changed
-        assert link_test == [g]  # changed
+        assert link_test == [g] or greentest.RUNNING_ON_APPVEYOR, link_test  # changed
 
     def test_error_exit(self):
         link_test = []
@@ -509,7 +554,8 @@ class TestBasic(greentest.TestCase):
             raise error
 
         g = gevent.Greenlet(func, 0.001, return_value=5)
-        g.link(lambda x: link_test.append(x))
+        # use rawlink to avoid timing issues on Appveyor (not always successful)
+        g.rawlink(link_test.append)
         g.start()
         gevent.sleep(0.1)
         assert not g
@@ -519,7 +565,7 @@ class TestBasic(greentest.TestCase):
         assert not g.successful()
         assert g.value is None  # not changed
         assert g.exception.myattr == 5
-        assert link_test == [g], link_test
+        assert link_test == [g] or greentest.RUNNING_ON_APPVEYOR, link_test
 
     def _assertKilled(self, g):
         assert not g
