@@ -1,4 +1,3 @@
-from __future__ import with_statement
 from time import time
 import gevent
 from gevent import pool
@@ -35,6 +34,22 @@ class TestCoroutinePool(unittest.TestCase):
         result = pool.apply(some_work)
         self.assertEqual(value, result)
 
+    def test_apply_raises(self):
+        pool = self.klass(1)
+
+        def raiser():
+            raise ExpectedException()
+        try:
+            pool.apply(raiser)
+        except ExpectedException:
+            pass
+        else:
+            self.fail("Should have raised ExpectedException")
+    # Don't let the metaclass automatically force any error
+    # that reaches the hub from a spawned greenlet to become
+    # fatal; that defeats the point of the test.
+    test_apply_raises.error_fatal = False
+
     def test_multiple_coros(self):
         evt = Event()
         results = []
@@ -78,6 +93,9 @@ class TestCoroutinePool(unittest.TestCase):
         evt.wait()
 
     def test_stderr_raising(self):
+        if greentest.PYPY:
+            # Does not work on PyPy
+            return
         # testing that really egregious errors in the error handling code
         # (that prints tracebacks to stderr) don't cause the pool to lose
         # any members
@@ -237,7 +255,7 @@ TIMEOUT1, TIMEOUT2, TIMEOUT3 = 0.082, 0.035, 0.14
 
 
 class TestPool(greentest.TestCase):
-    __timeout__ = 5
+    __timeout__ = 5 if not greentest.RUNNING_ON_APPVEYOR else 20
     size = 1
 
     def setUp(self):
@@ -261,14 +279,14 @@ class TestPool(greentest.TestCase):
         res = self.pool.apply_async(sqr, (7, TIMEOUT1,))
         get = TimingWrapper(res.get)
         self.assertEqual(get(), 49)
-        self.assertAlmostEqual(get.elapsed, TIMEOUT1, 1)
+        self.assertTimeoutAlmostEqual(get.elapsed, TIMEOUT1, 1)
 
     def test_async_callback(self):
         result = []
         res = self.pool.apply_async(sqr, (7, TIMEOUT1,), callback=lambda x: result.append(x))
         get = TimingWrapper(res.get)
         self.assertEqual(get(), 49)
-        self.assertAlmostEqual(get.elapsed, TIMEOUT1, 1)
+        self.assertTimeoutAlmostEqual(get.elapsed, TIMEOUT1, 1)
         gevent.sleep(0)  # let's the callback run
         assert result == [49], result
 
@@ -276,7 +294,7 @@ class TestPool(greentest.TestCase):
         res = self.pool.apply_async(sqr, (6, TIMEOUT2 + 0.2))
         get = TimingWrapper(res.get)
         self.assertRaises(gevent.Timeout, get, timeout=TIMEOUT2)
-        self.assertAlmostEqual(get.elapsed, TIMEOUT2, 1)
+        self.assertTimeoutAlmostEqual(get.elapsed, TIMEOUT2, 1)
         self.pool.join()
 
     def test_imap(self):
@@ -338,7 +356,7 @@ class TestPool(greentest.TestCase):
             expected = ['1', '2', '10']
         self.assertEqual(result, expected)
 
-    # https://github.com/surfly/gevent/issues/423
+    # https://github.com/gevent/gevent/issues/423
     def test_imap_no_stop(self):
         q = Queue()
         q.put(123)
@@ -353,7 +371,7 @@ class TestPool(greentest.TestCase):
         result = list(self.pool.imap_unordered(lambda _: _, q))
         self.assertEqual(result, [1234])
 
-    # same issue, but different test: https://github.com/surfly/gevent/issues/311
+    # same issue, but different test: https://github.com/gevent/gevent/issues/311
     def test_imap_final_sleep(self):
         result = list(self.pool.imap(sqr, final_sleep()))
         self.assertEqual(result, [0, 1, 4])
@@ -362,6 +380,37 @@ class TestPool(greentest.TestCase):
         result = list(self.pool.imap_unordered(sqr, final_sleep()))
         self.assertEqual(result, [0, 1, 4])
 
+    # Issue 638
+    def test_imap_unordered_bounded_queue(self):
+        iterable = list(range(100))
+
+        running = [0]
+
+        def short_running_func(i, j):
+            running[0] += 1
+            return i
+
+        # Send two iterables to make sure varargs and kwargs are handled
+        # correctly
+        for meth in self.pool.imap_unordered, self.pool.imap:
+            running[0] = 0
+            mapping = meth(short_running_func, iterable, iterable,
+                           maxsize=1)
+
+            # Simulate a long running reader. No matter how many workers
+            # we have, we will never have a queue more than size 1
+            def reader():
+                result = []
+                for i, x in enumerate(mapping):
+                    self.assertTrue(running[0] <= i + 2, running[0])
+                    result.append(x)
+                    gevent.sleep(0.01)
+                    self.assertTrue(len(mapping.queue) <= 2, len(mapping.queue))
+                return result
+
+            l = reader()
+            self.assertEqual(sorted(l), iterable)
+
 
 class TestPool2(TestPool):
     size = 2
@@ -379,6 +428,16 @@ class TestPoolUnlimit(TestPool):
     size = None
 
 
+class TestPool0(greentest.TestCase):
+    size = 0
+
+    def test_wait_full(self):
+        p = pool.Pool(size=0)
+        self.assertEqual(0, p.free_count())
+        self.assertTrue(p.full())
+        self.assertEqual(0, p.wait_available(timeout=0.01))
+
+
 class TestJoinSleep(greentest.GenericWaitTestCase):
 
     def wait(self, timeout):
@@ -419,7 +478,7 @@ class TestSpawn(greentest.TestCase):
         self.assertEqual(len(p), 1)
         p.spawn(gevent.sleep, 0.1)  # this spawn blocks until the old one finishes
         self.assertEqual(len(p), 1)
-        gevent.sleep(0.19)
+        gevent.sleep(0.19 if not greentest.RUNNING_ON_APPVEYOR else 0.5)
         self.assertEqual(len(p), 0)
 
 
@@ -461,18 +520,18 @@ class TestErrorInHandler(greentest.TestCase):
     def test_imap(self):
         p = pool.Pool(1)
         it = p.imap(divide_by, [1, 0, 2])
-        self.assertEqual(it.next(), 1.0)
-        self.assertRaises(ZeroDivisionError, it.next)
-        self.assertEqual(it.next(), 0.5)
-        self.assertRaises(StopIteration, it.next)
+        self.assertEqual(next(it), 1.0)
+        self.assertRaises(ZeroDivisionError, next, it)
+        self.assertEqual(next(it), 0.5)
+        self.assertRaises(StopIteration, next, it)
 
     def test_imap_unordered(self):
         p = pool.Pool(1)
         it = p.imap_unordered(divide_by, [1, 0, 2])
-        self.assertEqual(it.next(), 1.0)
-        self.assertRaises(ZeroDivisionError, it.next)
-        self.assertEqual(it.next(), 0.5)
-        self.assertRaises(StopIteration, it.next)
+        self.assertEqual(next(it), 1.0)
+        self.assertRaises(ZeroDivisionError, next, it)
+        self.assertEqual(next(it), 0.5)
+        self.assertRaises(StopIteration, next, it)
 
 
 if __name__ == '__main__':
