@@ -22,11 +22,20 @@
 """This test checks that underlying socket instances (gevent.socket.socket._sock)
 are not leaked by the hub.
 """
+from __future__ import print_function
 from _socket import socket
-
-
-class Socket(socket):
-    "Something we can have a weakref to"
+import sys
+if sys.version_info[0] >= 3:
+    # Python3 enforces that __weakref__ appears only once,
+    # and not when a slotted class inherits from an unslotted class.
+    # We mess around with the class MRO below and violate that rule
+    # (because socket.socket defines __slots__ with __weakref__),
+    # so import socket.socket before that can happen.
+    __import__('socket')
+    Socket = socket
+else:
+    class Socket(socket):
+        "Something we can have a weakref to"
 
 import _socket
 _socket.socket = Socket
@@ -35,7 +44,10 @@ import greentest
 from gevent import monkey; monkey.patch_all()
 
 from pprint import pformat
-from thread import start_new_thread
+try:
+    from thread import start_new_thread
+except ImportError:
+    from _thread import start_new_thread
 from time import sleep
 import weakref
 import gc
@@ -63,26 +75,26 @@ def handle_request(s, raise_on_timeout):
             raise
         else:
             return
-    #print 'handle_request - accepted'
+    #print('handle_request - accepted')
     res = conn.recv(100)
-    assert res == 'hello', repr(res)
-    #print 'handle_request - recvd %r' % res
-    res = conn.send('bye')
-    #print 'handle_request - sent %r' % res
-    #print 'handle_request - conn refcount: %s' % sys.getrefcount(conn)
+    assert res == b'hello', repr(res)
+    #print('handle_request - recvd %r' % res)
+    res = conn.send(b'bye')
+    #print('handle_request - sent %r' % res)
+    #print('handle_request - conn refcount: %s' % sys.getrefcount(conn))
     #conn.close()
 
 
 def make_request(port):
-    #print 'make_request'
+    #print('make_request')
     s = socket.socket()
     s.connect(('127.0.0.1', port))
-    #print 'make_request - connected'
-    res = s.send('hello')
-    #print 'make_request - sent %s' % res
+    #print('make_request - connected')
+    res = s.send(b'hello')
+    #print('make_request - sent %s' % res)
     res = s.recv(100)
-    assert res == 'bye', repr(res)
-    #print 'make_request - recvd %r' % res
+    assert res == b'bye', repr(res)
+    #print('make_request - recvd %r' % res)
     #s.close()
 
 
@@ -93,22 +105,38 @@ def run_interaction(run_client):
         port = s.getsockname()[1]
         start_new_thread(make_request, (port, ))
     sleep(0.1 + SOCKET_TIMEOUT)
-    #print sys.getrefcount(s._sock)
+    #print(sys.getrefcount(s._sock))
     #s.close()
-    return weakref.ref(s._sock)
+    w = weakref.ref(s._sock)
+    s.close()
+    if greentest.RUNNING_ON_APPVEYOR:
+        # The background thread may not have even had a chance to run
+        # yet, sleep again to be sure it does. Otherwise there could be
+        # strong refs to the socket still around.
+        try:
+            sleep(0.1 + SOCKET_TIMEOUT)
+        except Exception:
+            pass
+
+    return w
 
 
 def run_and_check(run_client):
     w = run_interaction(run_client=run_client)
+    if greentest.PYPY:
+        # PyPy doesn't use a strict ref counting and must
+        # run a gc, but the object should be gone
+        gc.collect()
     if w():
-        print pformat(gc.get_referrers(w()))
+        print(pformat(gc.get_referrers(w())))
         for x in gc.get_referrers(w()):
-            print pformat(x)
+            print(pformat(x))
             for y in gc.get_referrers(x):
-                print '-', pformat(y)
+                print('-', pformat(y))
         raise AssertionError('server should be dead by now')
 
 
+@greentest.skipOnAppVeyor("Often fail with timeouts; not sure why")
 class Test(greentest.TestCase):
 
     def test_clean_exit(self):
