@@ -1,3 +1,4 @@
+import six
 import sys
 import os
 from gevent import select, socket
@@ -22,6 +23,20 @@ if sys.platform != 'win32':
                 os.close(r)
                 os.close(w)
 
+    if hasattr(select, 'poll') and sys.platform != 'darwin':
+
+        class TestPollRead(greentest.GenericWaitTestCase):
+            def wait(self, timeout):
+                r, w = os.pipe()
+                try:
+                    poll = select.poll()
+                    poll.register(r)
+                    poll.poll(timeout * 1000)
+                    poll.unregister(r)
+                finally:
+                    os.close(r)
+                    os.close(w)
+
 
 class TestSelectTypes(greentest.TestCase):
 
@@ -29,14 +44,11 @@ class TestSelectTypes(greentest.TestCase):
         sock = socket.socket()
         select.select([int(sock.fileno())], [], [], 0.001)
 
-    try:
-        long
-    except NameError:
-        pass
-    else:
+    if hasattr(six.builtins, 'long'):
         def test_long(self):
             sock = socket.socket()
-            select.select([long(sock.fileno())], [], [], 0.001)
+            select.select(
+                [six.builtins.long(sock.fileno())], [], [], 0.001)
 
     def test_string(self):
         self.switch_expected = False
