@@ -1,7 +1,6 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
-from __future__ import with_statement
-import sys
+import six
 import re
 import greentest
 import socket
@@ -9,6 +8,7 @@ from time import time
 import gevent
 import gevent.socket as gevent_socket
 from util import log
+from six import xrange
 
 
 resolver = gevent.get_hub().resolver
@@ -29,8 +29,8 @@ def _run(function, *args):
         result = function(*args)
         assert not isinstance(result, BaseException), repr(result)
         return result
-    except Exception:
-        return sys.exc_info()[1]
+    except Exception as ex:
+        return ex
 
 
 def format_call(function, args):
@@ -86,15 +86,32 @@ def compare_relaxed(a, b):
     >>> compare_relaxed('2a00:1450:4016:800::1013', '2a00:1450:4008:c01::93')
     True
 
-    >>> compare_relaxed('a.surfly.com', 'b.surfly.com')
+    >>> compare_relaxed('2001:470::e852:4a38:9d7f:0', '2001:470:6d00:1c:1::d00')
     True
 
-    >>> compare_relaxed('a.surfly.com', 'a.gevent.org')
+    >>> compare_relaxed('2001:470:4147:4943:6161:6161:2e74:6573', '2001:470::')
+    True
+
+    >>> compare_relaxed('2607:f8b0:6708:24af:1fd:700:60d4:4af', '2607:f8b0:2d00::f000:0')
+    True
+
+    >>> compare_relaxed('a.google.com', 'b.google.com')
+    True
+
+    >>> compare_relaxed('a.google.com', 'a.gevent.org')
     False
     """
-    if a.count(':') == 5 and b.count(':') == 5:
-        # IPv6 address from different requests might be different
-        return True
+    # IPv6 address from different requests might be different
+    a_segments = a.count(':')
+    b_segments = b.count(':')
+    if a_segments and b_segments:
+        if a_segments == b_segments and a_segments in (4, 5, 6, 7):
+            return True
+        if a.rstrip(':').startswith(b.rstrip(':')) or b.rstrip(':').startswith(a.rstrip(':')):
+            return True
+        if a_segments >= 2 and b_segments >= 2 and a.split(':')[:2] == b.split(':')[:2]:
+            return True
+
     return a.split('.', 1)[-1] == b.split('.', 1)[-1]
 
 
@@ -123,7 +140,7 @@ def relaxed_is_equal(a, b):
         return False
     if a == b:
         return True
-    if isinstance(a, basestring):
+    if isinstance(a, six.string_types):
         return compare_relaxed(a, b)
     if len(a) != len(b):
         return False
@@ -211,19 +228,31 @@ class TestCase(greentest.TestCase):
 
         return gevent_result
 
+    def _normalize_result(self, result):
+        return result
+
     def assertEqualResults(self, real_result, gevent_result, func, args):
-        errors = [socket.gaierror, socket.herror, TypeError]
-        if type(real_result) in errors and type(gevent_result) in errors:
+        errors = (socket.gaierror, socket.herror, TypeError)
+        if isinstance(real_result, errors) and isinstance(gevent_result, errors):
             if type(real_result) is not type(gevent_result):
                 log('WARNING: error type mismatch: %r (gevent) != %r (stdlib)', gevent_result, real_result)
             return
+
+        real_result = self._normalize_result(real_result)
+        gevent_result = self._normalize_result(gevent_result)
+
         real_result_repr = repr(real_result)
         gevent_result_repr = repr(gevent_result)
         if real_result_repr == gevent_result_repr:
             return
         if relaxed_is_equal(gevent_result, real_result):
             return
-        raise AssertionError('%r != %r\n    %s' % (gevent_result, real_result, format_call(func, args)))
+
+        # From 2.7 on, assertEqual does a better job highlighting the results than we would
+        # because it calls assertSequenceEqual, which highlights the exact
+        # difference in the tuple
+        msg = format_call(func, args)
+        self.assertEqual((msg, gevent_result), (msg, real_result))
 
 
 class TestTypeError(TestCase):
@@ -289,9 +318,15 @@ for ip, host in re.findall(r'^\s*(\d+\.\d+\.\d+\.\d+)\s+([^\s]+)', etc_hosts, re
 
 
 class TestGeventOrg(TestCase):
-    pass
 
-add(TestGeventOrg, 'gevent.org')
+    HOSTNAME = 'www.gevent.org'
+
+# For this test to work correctly, it needs to resolve to
+# an address with a single A record; round-robin DNS and multiple A records
+# may mess it up (subsequent requests---and we always make two---may return
+# unequal results). We used to use gevent.org, but that now has multiple A records;
+# trying www.gevent.org which is a CNAME to readthedocs.org.
+add(TestGeventOrg, TestGeventOrg.HOSTNAME)
 
 
 class TestFamily(TestCase):
@@ -299,15 +334,15 @@ class TestFamily(TestCase):
     @classmethod
     def getresult(cls):
         if not hasattr(cls, '_result'):
-            cls._result = getattr(socket, 'getaddrinfo')('gevent.org', None)
+            cls._result = getattr(socket, 'getaddrinfo')(TestGeventOrg.HOSTNAME, None)
         return cls._result
 
     def assert_error(self, error, function, *args):
         try:
             result = function(*args)
             raise AssertionError('%s: Expected to raise %s, instead returned %r' % (function, error, result))
-        except Exception, ex:
-            if isinstance(error, basestring):
+        except Exception as ex:
+            if isinstance(error, six.string_types):
                 repr_error = error
             else:
                 repr_error = repr(error)
@@ -318,18 +353,18 @@ class TestFamily(TestCase):
             raise
 
     def test_inet(self):
-        self.assertEqual(gevent_socket.getaddrinfo('gevent.org', None, socket.AF_INET), self.getresult())
+        self.assertEqual(gevent_socket.getaddrinfo(TestGeventOrg.HOSTNAME, None, socket.AF_INET), self.getresult())
 
     def test_unspec(self):
-        self.assertEqual(gevent_socket.getaddrinfo('gevent.org', None, socket.AF_UNSPEC), self.getresult())
+        self.assertEqual(gevent_socket.getaddrinfo(TestGeventOrg.HOSTNAME, None, socket.AF_UNSPEC), self.getresult())
 
     def test_badvalue(self):
-        self._test('getaddrinfo', 'gevent.org', None, 255)
-        self._test('getaddrinfo', 'gevent.org', None, 255000)
-        self._test('getaddrinfo', 'gevent.org', None, -1)
+        self._test('getaddrinfo', TestGeventOrg.HOSTNAME, None, 255)
+        self._test('getaddrinfo', TestGeventOrg.HOSTNAME, None, 255000)
+        self._test('getaddrinfo', TestGeventOrg.HOSTNAME, None, -1)
 
     def test_badtype(self):
-        self._test('getaddrinfo', 'gevent.org', 'x')
+        self._test('getaddrinfo', TestGeventOrg.HOSTNAME, 'x')
 
 
 class Test_getaddrinfo(TestCase):
@@ -338,16 +373,16 @@ class Test_getaddrinfo(TestCase):
         self._test('getaddrinfo', *args)
 
     def test_80(self):
-        self._test_getaddrinfo('gevent.org', 80)
+        self._test_getaddrinfo(TestGeventOrg.HOSTNAME, 80)
 
     def test_int_string(self):
-        self._test_getaddrinfo('gevent.org', '80')
+        self._test_getaddrinfo(TestGeventOrg.HOSTNAME, '80')
 
     def test_0(self):
-        self._test_getaddrinfo('gevent.org', 0)
+        self._test_getaddrinfo(TestGeventOrg.HOSTNAME, 0)
 
     def test_http(self):
-        self._test_getaddrinfo('gevent.org', 'http')
+        self._test_getaddrinfo(TestGeventOrg.HOSTNAME, 'http')
 
     def test_notexistent_tld(self):
         self._test_getaddrinfo('myhost.mytld', 53)
@@ -356,10 +391,10 @@ class Test_getaddrinfo(TestCase):
         self._test_getaddrinfo('sdfsdfgu5e66098032453245wfdggd.com', 80)
 
     def test1(self):
-        return self._test_getaddrinfo('gevent.org', 52, socket.AF_UNSPEC, socket.SOCK_STREAM, 0, 0)
+        return self._test_getaddrinfo(TestGeventOrg.HOSTNAME, 52, socket.AF_UNSPEC, socket.SOCK_STREAM, 0, 0)
 
     def test2(self):
-        return self._test_getaddrinfo('gevent.org', 53, socket.AF_INET, socket.SOCK_DGRAM, 17)
+        return self._test_getaddrinfo(TestGeventOrg.HOSTNAME, 53, socket.AF_INET, socket.SOCK_DGRAM, 17)
 
     def test3(self):
         return self._test_getaddrinfo('google.com', 'http', socket.AF_INET6)
@@ -431,14 +466,14 @@ class Test_getnameinfo_127001(TestCase):
 class Test_getnameinfo_geventorg(TestCase):
 
     def test_NUMERICHOST(self):
-        self._test('getnameinfo', ('gevent.org', 80), 0)
-        self._test('getnameinfo', ('gevent.org', 80), socket.NI_NUMERICHOST)
+        self._test('getnameinfo', (TestGeventOrg.HOSTNAME, 80), 0)
+        self._test('getnameinfo', (TestGeventOrg.HOSTNAME, 80), socket.NI_NUMERICHOST)
 
     def test_NUMERICSERV(self):
-        self._test('getnameinfo', ('gevent.org', 80), socket.NI_NUMERICSERV)
+        self._test('getnameinfo', (TestGeventOrg.HOSTNAME, 80), socket.NI_NUMERICSERV)
 
     def test_domain1(self):
-        self._test('getnameinfo', ('gevent.org', 80), 0)
+        self._test('getnameinfo', (TestGeventOrg.HOSTNAME, 80), 0)
 
     def test_domain2(self):
         self._test('getnameinfo', ('www.gevent.org', 80), 0)
