@@ -1,5 +1,7 @@
 import greentest
 from gevent import socket
+import errno
+import sys
 
 
 class TestClosedSocket(greentest.TestCase):
@@ -10,10 +12,17 @@ class TestClosedSocket(greentest.TestCase):
         sock = socket.socket()
         sock.close()
         try:
-            sock.send('a', timeout=1)
-        except socket.error, ex:
-            if ex[0] != 9:
-                raise
+            sock.send(b'a', timeout=1)
+            raise AssertionError("Should not get here")
+        except (socket.error, OSError) as ex:
+            if ex.args[0] != errno.EBADF:
+                if sys.platform.startswith('win'):
+                    # Windows/Py3 raises "OSError: [WinError 10038] "
+                    # which is not standard and not what it does
+                    # on Py2.
+                    pass
+                else:
+                    raise
 
 
 class TestRef(greentest.TestCase):
@@ -27,6 +36,7 @@ class TestRef(greentest.TestCase):
         assert sock.ref is False, sock.ref
         assert sock._read_event.ref is False, sock.ref
         assert sock._write_event.ref is False, sock.ref
+        sock.close()
 
 
 if __name__ == '__main__':
