@@ -1,4 +1,3 @@
-from __future__ import with_statement
 import sys
 from time import time, sleep
 import random
@@ -6,7 +5,12 @@ import weakref
 import greentest
 from gevent.threadpool import ThreadPool
 import gevent
+from greentest import ExpectedException
 import six
+import gc
+
+
+PYPY = hasattr(sys, 'pypy_version_info')
 
 
 class TestCase(greentest.TestCase):
@@ -44,6 +48,23 @@ class PoolBasicTests(TestCase):
         result = pool.apply(lambda a: ('foo', a), (1, ))
         self.assertEqual(result, ('foo', 1))
 
+    def test_apply_raises(self):
+        self.pool = pool = ThreadPool(1)
+
+        def raiser():
+            raise ExpectedException()
+
+        try:
+            pool.apply(raiser)
+        except ExpectedException:
+            pass
+        else:
+            self.fail("Should have raised ExpectedException")
+    # Don't let the metaclass automatically force any error
+    # that reaches the hub from a spawned greenlet to become
+    # fatal; that defeats the point of the test.
+    test_apply_raises.error_fatal = False
+
     def test_init_valueerror(self):
         self.switch_expected = False
         self.assertRaises(ValueError, ThreadPool, -1)
@@ -134,6 +155,22 @@ class TestPool(TestCase):
             self.assertEqual(six.advance_iterator(it), i * i)
         self.assertRaises(StopIteration, lambda: six.advance_iterator(it))
 
+    def test_imap_gc(self):
+        it = self.pool.imap(sqr, range(10))
+        for i in range(10):
+            self.assertEqual(six.advance_iterator(it), i * i)
+            gc.collect()
+        self.assertRaises(StopIteration, lambda: six.advance_iterator(it))
+
+    def test_imap_unordered_gc(self):
+        it = self.pool.imap_unordered(sqr, range(10))
+        result = []
+        for i in range(10):
+            result.append(six.advance_iterator(it))
+            gc.collect()
+        self.assertRaises(StopIteration, lambda: six.advance_iterator(it))
+        self.assertEqual(sorted(result), [x * x for x in range(10)])
+
     def test_imap_random(self):
         it = self.pool.imap(sqr_random_sleep, range(10))
         self.assertEqual(list(it), list(map(sqr, range(10))))
@@ -175,6 +212,25 @@ class TestPool(TestCase):
 class TestPool2(TestPool):
     size = 2
 
+    def test_recursive_apply(self):
+        p = self.pool
+
+        def a():
+            return p.apply(b)
+
+        def b():
+            # make sure we can do both types of callbacks
+            # (loop iteration and end-of-loop) in the recursive
+            # call
+            gevent.sleep()
+            gevent.sleep(0.001)
+            return "B"
+
+        result = p.apply(a)
+        self.assertEqual(result, "B")
+    # Asking for the hub in the new thread shows up as a "leak"
+    test_recursive_apply.ignore_leakcheck = True
+
 
 class TestPool3(TestPool):
     size = 3
@@ -329,8 +385,11 @@ class TestRef(TestCase):
 
             refs.append(weakref.ref(func))
             del func, result
+            if PYPY:
+                gc.collect()
+                gc.collect()
             for index, r in enumerate(refs):
-                assert r() is None, (index, r(), sys.getrefcount(r()), refs)
+                assert r() is None, (index, r(), greentest.getrefcount(r()), refs)
             assert len(refs) == 4, refs
 
 
