@@ -1,5 +1,6 @@
 # testing gevent's Event, Lock, RLock, Semaphore, BoundedSemaphore with standard test_threading
-from __future__ import with_statement
+from __future__ import print_function
+from six import xrange
 
 setup_ = '''from gevent import monkey; monkey.patch_all()
 from gevent.event import Event
@@ -8,35 +9,38 @@ from gevent.thread import allocate_lock as Lock
 import threading
 threading.Event = Event
 threading.Lock = Lock
+# NOTE: We're completely patching around the allocate_lock
+# patch we try to do with RLock; our monkey patch doesn't
+# behave this way, but we do it in tests to make sure that
+# our RLock implementation behaves correctly by itself.
+# However, we must test the patched version too, so make it
+# available.
+threading.NativeRLock = threading.RLock
 threading.RLock = RLock
 threading.Semaphore = Semaphore
 threading.BoundedSemaphore = BoundedSemaphore
-if not hasattr(threading, 'current_thread'):
-    threading.current_thread = threading.currentThread
-if not hasattr(threading.Thread, 'name'):
-    threading.Thread.name = property(lambda self: self.getName())
-if not hasattr(threading.Thread, 'is_alive'):
-    threading.Thread.is_alive = threading.Thread.isAlive
-if not hasattr(threading.Thread, 'daemon'):
-    threading.Thread.daemon = property(threading.Thread.isDaemon, threading.Thread.setDaemon)
-if not hasattr(threading._Condition, 'notify_all'):
-    threading._Condition.notify_all = threading._Condition.notifyAll
 '''
 
-exec setup_
+exec(setup_)
 
 setup_3 = '\n'.join('            %s' % line for line in setup_.split('\n'))
 setup_4 = '\n'.join('                %s' % line for line in setup_.split('\n'))
-setup_5 = '\n'.join('                    %s' % line for line in setup_.split('\n'))
 
 
-import test.test_support
-from test.test_support import verbose
+try:
+    from test import support
+    from test.support import verbose
+except ImportError:
+    from test import test_support as support
+    from test.test_support import verbose
 import random
 import re
 import sys
 import threading
-import thread
+try:
+    import thread
+except ImportError:
+    import _thread as thread
 import time
 import unittest
 import weakref
@@ -71,26 +75,26 @@ class TestThread(threading.Thread):
     def run(self):
         delay = random.random() / 10000.0
         if verbose:
-            print 'task %s will run for %.1f usec' % (
-                self.name, delay * 1e6)
+            print('task %s will run for %.1f usec' % (
+                self.name, delay * 1e6))
 
         with self.sema:
             with self.mutex:
                 self.nrunning.inc()
                 if verbose:
-                    print self.nrunning.get(), 'tasks are running'
+                    print(self.nrunning.get(), 'tasks are running')
                 self.testcase.assert_(self.nrunning.get() <= 3)
 
             time.sleep(delay)
             if verbose:
-                print 'task', self.name, 'done'
+                print('task', self.name, 'done')
 
             with self.mutex:
                 self.nrunning.dec()
                 self.testcase.assert_(self.nrunning.get() >= 0)
                 if verbose:
-                    print '%s is finished. %d tasks are running' % (
-                        self.name, self.nrunning.get())
+                    print('%s is finished. %d tasks are running' % (
+                        self.name, self.nrunning.get()))
 
 
 class ThreadTests(unittest.TestCase):
@@ -112,49 +116,59 @@ class ThreadTests(unittest.TestCase):
         for i in range(NUMTASKS):
             t = TestThread("<thread %d>" % i, self, sema, mutex, numrunning)
             threads.append(t)
+            t.daemon = False # Under PYPY we get daemon by default?
             if hasattr(t, 'ident'):
                 self.failUnlessEqual(t.ident, None)
-                self.assert_(re.match('<TestThread\(.*, initial\)>', repr(t)))
+                self.assertFalse(t.daemon)
+                self.assert_(re.match(r'<TestThread\(.*, initial\)>', repr(t)))
             t.start()
 
         if verbose:
-            print 'waiting for all tasks to complete'
+            print('waiting for all tasks to complete')
         for t in threads:
             t.join(NUMTASKS)
             self.assert_(not t.is_alive())
             if hasattr(t, 'ident'):
                 self.failIfEqual(t.ident, 0)
                 self.assertFalse(t.ident is None)
-                self.assert_(re.match('<TestThread\(.*, \w+ -?\d+\)>', repr(t)))
+                self.assert_(re.match(r'<TestThread\(.*, \w+ -?\d+\)>', repr(t)))
         if verbose:
-            print 'all tasks done'
+            print('all tasks done')
         self.assertEqual(numrunning.get(), 0)
 
-    if sys.version_info[:2] > (2, 5):
-        def test_ident_of_no_threading_threads(self):
-            # The ident still must work for the main thread and dummy threads.
-            self.assertFalse(threading.currentThread().ident is None)
+    def test_ident_of_no_threading_threads(self):
+        # The ident still must work for the main thread and dummy threads,
+        # as must the repr and str.
 
-            def f():
-                ident.append(threading.currentThread().ident)
-                done.set()
-            done = threading.Event()
-            ident = []
-            thread.start_new_thread(f, ())
-            done.wait()
-            self.assertFalse(ident[0] is None)
-            # Kill the "immortal" _DummyThread
-            del threading._active[ident[0]]
+        t = threading.currentThread()
+        self.assertFalse(t.ident is None)
+        str(t)
+        repr(t)
+
+        def f():
+            t = threading.currentThread()
+            ident.append(t.ident)
+            str(t)
+            repr(t)
+            done.set()
+
+        done = threading.Event()
+        ident = []
+        thread.start_new_thread(f, ())
+        done.wait()
+        self.assertFalse(ident[0] is None)
+        # Kill the "immortal" _DummyThread
+        del threading._active[ident[0]]
 
     # run with a small(ish) thread stack size (256kB)
     def test_various_ops_small_stack(self):
         if verbose:
-            print 'with 256kB thread stack size...'
+            print('with 256kB thread stack size...')
         try:
             threading.stack_size(262144)
         except thread.error:
             if verbose:
-                print 'platform does not support changing thread stack size'
+                print('platform does not support changing thread stack size')
             return
         self.test_various_ops()
         threading.stack_size(0)
@@ -162,12 +176,12 @@ class ThreadTests(unittest.TestCase):
     # run with a large thread stack size (1MB)
     def test_various_ops_large_stack(self):
         if verbose:
-            print 'with 1MB thread stack size...'
+            print('with 1MB thread stack size...')
         try:
             threading.stack_size(0x100000)
         except thread.error:
             if verbose:
-                print 'platform does not support changing thread stack size'
+                print('platform does not support changing thread stack size')
             return
         self.test_various_ops()
         threading.stack_size(0)
@@ -198,7 +212,7 @@ class ThreadTests(unittest.TestCase):
             import ctypes
         except ImportError:
             if verbose:
-                print "test_PyThreadState_SetAsyncExc can't import ctypes"
+                print("test_PyThreadState_SetAsyncExc can't import ctypes")
             return  # can't do anything
 
         set_async_exc = ctypes.pythonapi.PyThreadState_SetAsyncExc
@@ -232,120 +246,124 @@ class ThreadTests(unittest.TestCase):
         t.daemon = True  # so if this fails, we don't hang Python at shutdown
         t.start()
         if verbose:
-            print "    started worker thread"
+            print("    started worker thread")
 
         # Try a thread id that doesn't make sense.
         if verbose:
-            print "    trying nonsensical thread id"
+            print("    trying nonsensical thread id")
         result = set_async_exc(ctypes.c_long(-1), exception)
         self.assertEqual(result, 0)  # no thread states modified
 
         # Now raise an exception in the worker thread.
         if verbose:
-            print "    waiting for worker thread to get started"
+            print("    waiting for worker thread to get started")
         worker_started.wait()
         if verbose:
-            print "    verifying worker hasn't exited"
+            print("    verifying worker hasn't exited")
         self.assert_(not t.finished)
         if verbose:
-            print "    attempting to raise asynch exception in worker"
+            print("    attempting to raise asynch exception in worker")
         result = set_async_exc(ctypes.c_long(t.id), exception)
         self.assertEqual(result, 1)  # one thread state modified
         if verbose:
-            print "    waiting for worker to say it caught the exception"
+            print("    waiting for worker to say it caught the exception")
         worker_saw_exception.wait(timeout=10)
         self.assert_(t.finished)
         if verbose:
-            print "    all OK -- joining worker"
+            print("    all OK -- joining worker")
         if t.finished:
             t.join()
         # else the thread is still running, and we have no way to kill it
 
-    if sys.version_info[:2] > (2, 5):
-        def test_limbo_cleanup(self):
-            # Issue 7481: Failure to start thread should cleanup the limbo map.
-            def fail_new_thread(*args):
-                raise thread.error()
-            _start_new_thread = threading._start_new_thread
-            threading._start_new_thread = fail_new_thread
-            try:
-                t = threading.Thread(target=lambda: None)
-                self.assertRaises(thread.error, t.start)
-                self.assertFalse(
-                    t in threading._limbo,
-                    "Failed to cleanup _limbo map on failure of Thread.start().")
-            finally:
-                threading._start_new_thread = _start_new_thread
+    def test_limbo_cleanup(self):
+        # Issue 7481: Failure to start thread should cleanup the limbo map.
+        def fail_new_thread(*args):
+            raise thread.error()
+        _start_new_thread = threading._start_new_thread
+        threading._start_new_thread = fail_new_thread
+        try:
+            t = threading.Thread(target=lambda: None)
+            self.assertRaises(thread.error, t.start)
+            self.assertFalse(
+                t in threading._limbo,
+                "Failed to cleanup _limbo map on failure of Thread.start().")
+        finally:
+            threading._start_new_thread = _start_new_thread
 
-    if sys.version_info[:2] > (2, 5):
-        def test_finalize_runnning_thread(self):
-            # Issue 1402: the PyGILState_Ensure / _Release functions may be called
-            # very late on python exit: on deallocation of a running thread for
-            # example.
+    def test_finalize_runnning_thread(self):
+        # Issue 1402: the PyGILState_Ensure / _Release functions may be called
+        # very late on python exit: on deallocation of a running thread for
+        # example.
+        try:
+            import ctypes
+            getattr(ctypes, 'pythonapi') # not available on PyPy
+        except (ImportError,AttributeError):
+            if verbose:
+                print("test_finalize_with_runnning_thread can't import ctypes")
+            return  # can't do anything
+
+        del ctypes  # pyflakes fix
+
+        import subprocess
+        rc = subprocess.call([sys.executable, "-c", """if 1:
+%s
+            import ctypes, sys, time
             try:
-                import ctypes
+                import thread
             except ImportError:
-                if verbose:
-                    print("test_finalize_with_runnning_thread can't import ctypes")
-                return  # can't do anything
+                import _thread as thread # Py3
 
-            del ctypes  # pyflakes fix
+            # This lock is used as a simple event variable.
+            ready = thread.allocate_lock()
+            ready.acquire()
 
-            import subprocess
-            rc = subprocess.call([sys.executable, "-c", """if 1:
+            # Module globals are cleared before __del__ is run
+            # So we save the functions in class dict
+            class C:
+                ensure = ctypes.pythonapi.PyGILState_Ensure
+                release = ctypes.pythonapi.PyGILState_Release
+                def __del__(self):
+                    state = self.ensure()
+                    self.release(state)
+
+            def waitingThread():
+                x = C()
+                ready.release()
+                time.sleep(100)
+
+            thread.start_new_thread(waitingThread, ())
+            ready.acquire()  # Be sure the other thread is waiting.
+            sys.exit(42)
+            """ % setup_3])
+        self.assertEqual(rc, 42)
+
+    def test_join_nondaemon_on_shutdown(self):
+        # Issue 1722344
+        # Raising SystemExit skipped threading._shutdown
+        import subprocess
+        p = subprocess.Popen([sys.executable, "-c", """if 1:
 %s
-                import ctypes, sys, time, thread
+                import threading
+                from time import sleep
 
-                # This lock is used as a simple event variable.
-                ready = thread.allocate_lock()
-                ready.acquire()
+                def child():
+                    sleep(1)
+                    # As a non-daemon thread we SHOULD wake up and nothing
+                    # should be torn down yet
+                    print("Woke up, sleep function is: %%r" %% sleep)
 
-                # Module globals are cleared before __del__ is run
-                # So we save the functions in class dict
-                class C:
-                    ensure = ctypes.pythonapi.PyGILState_Ensure
-                    release = ctypes.pythonapi.PyGILState_Release
-                    def __del__(self):
-                        state = self.ensure()
-                        self.release(state)
-
-                def waitingThread():
-                    x = C()
-                    ready.release()
-                    time.sleep(100)
-
-                thread.start_new_thread(waitingThread, ())
-                ready.acquire()  # Be sure the other thread is waiting.
-                sys.exit(42)
-                """ % setup_4])
-            self.assertEqual(rc, 42)
-
-    if sys.version_info[:2] > (2, 5):
-        def test_join_nondaemon_on_shutdown(self):
-            # Issue 1722344
-            # Raising SystemExit skipped threading._shutdown
-            import subprocess
-            p = subprocess.Popen([sys.executable, "-c", """if 1:
-%s
-                    import threading
-                    from time import sleep
-
-                    def child():
-                        sleep(1)
-                        # As a non-daemon thread we SHOULD wake up and nothing
-                        # should be torn down yet
-                        print "Woke up, sleep function is:", sleep
-
-                    threading.Thread(target=child).start()
-                    raise SystemExit
-                """ % setup_5],
-                stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE)
-            stdout, stderr = p.communicate()
-            stdout = stdout.strip()
-            assert re.match('^Woke up, sleep function is: <.*?sleep.*?>$', stdout), repr(stdout)
-            stderr = re.sub(r"^\[\d+ refs\]", "", stderr, re.MULTILINE).strip()
-            self.assertEqual(stderr, "")
+                threading.Thread(target=child).start()
+                raise SystemExit
+            """ % setup_4],
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE)
+        stdout, stderr = p.communicate()
+        stdout = stdout.strip()
+        stdout = stdout.decode('utf-8')
+        stderr = stderr.decode('utf-8')
+        assert re.match('^Woke up, sleep function is: <.*?sleep.*?>$', stdout), repr(stdout)
+        stderr = re.sub(r"^\[\d+ refs\]", "", stderr, re.MULTILINE).strip()
+        self.assertEqual(stderr, "")
 
     def test_enumerate_after_join(self):
         # Try hard to trigger #1703448: a thread is still returned in
@@ -366,7 +384,7 @@ class ThreadTests(unittest.TestCase):
         finally:
             sys.setcheckinterval(old_interval)
 
-    if sys.version_info[:2] > (2, 5):
+    if not hasattr(sys, 'pypy_version_info'):
         def test_no_refcycle_through_target(self):
             class RunSelfFunction(object):
                 def __init__(self, should_raise):
@@ -405,20 +423,19 @@ class ThreadJoinOnShutdown(unittest.TestCase):
         script = """if 1:
 %s
             import sys, os, time, threading
-
             # a thread, which waits for the main program to terminate
             def joiningfunc(mainthread):
                 mainthread.join()
-                print 'end of thread'
+                print('end of thread')
         \n""" % setup_3 + script
 
         import subprocess
         p = subprocess.Popen([sys.executable, "-c", script], stdout=subprocess.PIPE)
         rc = p.wait()
-        data = p.stdout.read().replace('\r', '')
-        self.assertEqual(data, "end of main\nend of thread\n")
-        self.failIf(rc == 2, "interpreter was blocked")
-        self.failUnless(rc == 0, "Unexpected error")
+        data = p.stdout.read().replace(b'\r', b'')
+        self.assertEqual(data, b"end of main\nend of thread\n")
+        self.failIf(rc == 2, b"interpreter was blocked")
+        self.failUnless(rc == 0, b"Unexpected error")
 
     def test_1_join_on_shutdown(self):
         # The usual case: on exit, wait for a non-daemon thread
@@ -428,7 +445,7 @@ class ThreadJoinOnShutdown(unittest.TestCase):
                                  args=(threading.current_thread(),))
             t.start()
             time.sleep(0.1)
-            print 'end of main'
+            print('end of main')
             """
         self._run_and_join(script)
 
@@ -446,7 +463,7 @@ class ThreadJoinOnShutdown(unittest.TestCase):
             t = threading.Thread(target=joiningfunc,
                                  args=(threading.current_thread(),))
             t.start()
-            print 'end of main'
+            print('end of main')
             """
         self._run_and_join(script)
 
@@ -460,8 +477,8 @@ class ThreadJoinOnShutdown(unittest.TestCase):
         # See http://bugs.python.org/issue3863.
         # skip disable because I think the bug shouldn't apply to gevent -- denis
         #if sys.platform in ('freebsd4', 'freebsd5', 'freebsd6', 'os2emx'):
-        #    print >>sys.stderr, ('Skipping test_3_join_in_forked_from_thread'
-        #                         ' due to known OS bugs on'), sys.platform
+        #    print(('Skipping test_3_join_in_forked_from_thread'
+        #          ' due to known OS bugs on'), sys.platform, file=sys.stderr)
         #    return
         script = """if 1:
             main_thread = threading.current_thread()
@@ -473,7 +490,7 @@ class ThreadJoinOnShutdown(unittest.TestCase):
 
                 t = threading.Thread(target=joiningfunc,
                                      args=(main_thread,))
-                print 'end of main'
+                print('end of main')
                 t.start()
                 t.join() # Should not block: main_thread is already stopped
 
@@ -512,6 +529,10 @@ class LockTests(lock_tests.LockTests):
 class RLockTests(lock_tests.RLockTests):
     locktype = staticmethod(threading.RLock)
 
+class NativeRLockTests(lock_tests.RLockTests):
+    # See comments at the top of the file for the difference
+    # between this and RLockTests, and why they both matter
+    locktype = staticmethod(threading.NativeRLock)
 
 class EventTests(lock_tests.EventTests):
     eventtype = staticmethod(threading.Event)
@@ -535,13 +556,14 @@ class BoundedSemaphoreTests(lock_tests.BoundedSemaphoreTests):
 
 
 def main():
-    test.test_support.run_unittest(LockTests, RLockTests, EventTests,
-                                   ConditionAsRLockTests, ConditionTests,
-                                   SemaphoreTests, BoundedSemaphoreTests,
-                                   ThreadTests,
-                                   ThreadJoinOnShutdown,
-                                   ThreadingExceptionTests,
-                                   )
+    support.run_unittest(LockTests, RLockTests, EventTests,
+                         ConditionAsRLockTests, ConditionTests,
+                         SemaphoreTests, BoundedSemaphoreTests,
+                         ThreadTests,
+                         ThreadJoinOnShutdown,
+                         ThreadingExceptionTests,
+                         NativeRLockTests,
+                         )
 
 if __name__ == "__main__":
     main()
