@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+from __future__ import print_function
 import six
 import sys
 import os
@@ -13,7 +14,7 @@ from util import log
 
 
 TIMEOUT = 180
-NWORKERS = int(os.environ.get('NWORKERS') or 8)
+NWORKERS = int(os.environ.get('NWORKERS') or 4)
 
 
 # tests that don't do well when run on busy box
@@ -22,8 +23,17 @@ RUN_ALONE = [
     'test__examples.py'
 ]
 
+# tests that can't be run when coverage is enabled
+IGNORE_COVERAGE = [
+    # Hangs forever
+    'test__threading_vs_settrace.py',
+    # XXX ?
+    'test__issue302monkey.py',
+    "test_subprocess.py",
+]
 
-def run_many(tests, expected=None, failfast=False):
+
+def run_many(tests, expected=(), failfast=False):
     global NWORKERS
     start = time.time()
     total = 0
@@ -105,30 +115,46 @@ def run_many(tests, expected=None, failfast=False):
     report(total, failed, passed, took=time.time() - start, expected=expected)
 
 
-def discover(tests=None, ignore=None):
-    if isinstance(ignore, basestring):
+def discover(tests=None, ignore=(), coverage=False):
+    if isinstance(ignore, six.string_types):
         ignore = load_list_from_file(ignore)
 
-    ignore = set(ignore or [])
+    ignore = set(ignore or ())
+    if coverage:
+        ignore.update(IGNORE_COVERAGE)
 
     if not tests:
-        tests = set(glob.glob('test_*.py'))
-        if ignore:
-            tests -= ignore
-        tests = sorted(tests)
+        tests = set(glob.glob('test_*.py')) - set(['test_support.py'])
+    else:
+        tests = set(tests)
+
+    if ignore:
+        # Always ignore the designated list, even if tests were specified
+        # on the command line. This fixes a nasty interaction with test__threading_vs_settrace.py
+        # being run under coverage when 'grep -l subprocess test*py' is used to list the tests
+        # to run.
+        tests -= ignore
+    tests = sorted(tests)
 
     to_process = []
     default_options = {'timeout': TIMEOUT}
 
     for filename in tests:
-        if 'TESTRUNNER' in open(filename).read():
+        with open(filename, 'rb') as f:
+            # Some of the test files (e.g., test__socket_dns) are
+            # UTF8 encoded. Depending on the environment, Python 3 may
+            # try to decode those as ASCII, which fails with UnicodeDecodeError.
+            # Thus, be sure to open and compare in binary mode.
+            contents = f.read()
+        if b'TESTRUNNER' in contents: # test__monkey_patching.py
             module = __import__(filename.rsplit('.', 1)[0])
             for cmd, options in module.TESTRUNNER():
                 if remove_options(cmd)[-1] in ignore:
                     continue
                 to_process.append((cmd, options))
         else:
-            to_process.append(([sys.executable, '-u', filename], default_options.copy()))
+            cmd = [sys.executable, '-u', filename]
+            to_process.append((cmd, default_options.copy()))
 
     return to_process
 
@@ -234,17 +260,29 @@ def main():
     parser.add_option('--full', action='store_true')
     parser.add_option('--config')
     parser.add_option('--failfast', action='store_true')
+    parser.add_option("--coverage", action="store_true")
     options, args = parser.parse_args()
     FAILING_TESTS = []
+    coverage = False
+    if options.coverage or os.environ.get("GEVENTTEST_COVERAGE"):
+        coverage = True
+        # NOTE: This must be run from the greentest directory
+        os.environ['COVERAGE_PROCESS_START'] = os.path.abspath(".coveragerc")
+        os.environ['PYTHONPATH'] = os.path.abspath("coveragesite") + os.pathsep + os.environ.get("PYTHONPATH", "")
+        # We change directory often, use an absolute path to keep all the
+        # coverage files (which will have distinct suffixes because of parallel=true in .coveragerc
+        # in this directory; makes them easier to combine and use with coverage report)
+        os.environ['COVERAGE_FILE'] = os.path.abspath(".") + os.sep + ".coverage"
+        print("Enabling coverage to", os.environ['COVERAGE_FILE'])
     if options.config:
         config = {}
         six.exec_(open(options.config).read(), config)
         FAILING_TESTS = config['FAILING_TESTS']
-    tests = discover(args, options.ignore)
+    tests = discover(args, options.ignore, coverage)
     if options.discover:
         for cmd, options in tests:
-            print (util.getname(cmd, env=options.get('env'), setenv=options.get('setenv')))
-        print ('%s tests found.' % len(tests))
+            print(util.getname(cmd, env=options.get('env'), setenv=options.get('setenv')))
+        print('%s tests found.' % len(tests))
     else:
         run_many(tests, expected=FAILING_TESTS, failfast=options.failfast)
 
