@@ -1,6 +1,6 @@
-from __future__ import with_statement
 import sys
 import os
+import six
 import traceback
 import unittest
 import threading
@@ -47,10 +47,10 @@ def killpg(pid):
         return
     try:
         return os.killpg(pid, 9)
-    except OSError, ex:
+    except OSError as ex:
         if ex.errno != 3:
             log('killpg(%r, 9) failed: %s: %s', pid, type(ex).__name__, ex)
-    except Exception, ex:
+    except Exception as ex:
         log('killpg(%r, 9) failed: %s: %s', pid, type(ex).__name__, ex)
 
 
@@ -65,7 +65,7 @@ def _kill(popen):
     if hasattr(popen, 'kill'):
         try:
             popen.kill()
-        except OSError, ex:
+        except OSError as ex:
             if ex.errno == 3:  # No such process
                 return
             if ex.errno == 13:  # Permission denied (translated from windows error 5: "Access is denied")
@@ -111,7 +111,7 @@ def getname(command, env=None, setenv=None):
         if key.startswith('GEVENT_') or key.startswith('GEVENTARES_'):
             result.append('%s=%s' % (key, value))
 
-    if isinstance(command, basestring):
+    if isinstance(command, six.string_types):
         result.append(command)
     else:
         result.extend(command)
@@ -157,8 +157,12 @@ class RunResult(object):
         self.output = output
         self.name = name
 
-    def __nonzero__(self):
-        return bool(self.code)
+    if six.PY3:
+        def __bool__(self):
+            return bool(self.code)
+    else:
+        def __nonzero__(self):
+            return bool(self.code)
 
     def __int__(self):
         return self.code
@@ -206,15 +210,31 @@ def run(command, **kwargs):
 class TestServer(unittest.TestCase):
     cwd = '../examples/'
     args = []
-    before_delay = 1
+    before_delay = 3
     after_delay = 0.5
+    popen = None
+    server = None # subclasses define this to be the path to the server.py
+    start_kwargs = None
+
+    def start(self):
+        kwargs = self.start_kwargs or {}
+        return start([sys.executable, '-u', self.server] + self.args, cwd=self.cwd, **kwargs)
+
+    def running_server(self):
+        from contextlib import contextmanager
+
+        @contextmanager
+        def running_server():
+            with self.start() as popen:
+                self.popen = popen
+                self.before()
+                yield
+                self.after()
+        return running_server()
 
     def test(self):
-        with start([sys.executable, '-u', self.server] + self.args, cwd=self.cwd) as popen:
-            self.popen = popen
-            self.before()
+        with self.running_server():
             self._run_all_tests()
-            self.after()
 
     def before(self):
         if self.before_delay is not None:
